#################################################################################
# Copyright (C) 2004 Mayetri Gupta. All rights reserved.                        #
# This program is free software; you can redistribute it and/or modify it under #
# the same terms as Perl itself.                                                #
#                                                                               #
# E-mail: gupta@bios.unc.edu                                                    #
#                                                                               #
# Program to extract unique motifs and sites from BioProspector output and      #
# compile a list in the required input format for the EMC Module Sampler        #
#                                                                               #
# IMPORTANT NOTE: ONLY works for Bioprospector.linux (2004) output format       #
#################################################################################

#!/usr/local/bin/perl

##
## Usage: extractBPmotifs.pl bp.out width emcms.in
##
## bp.out : output file from bioprospector
## width: width of motif as found by BioProspector
## emcms.in: output file for this program = input motif list file for EMC Module Sampler
##


$beforestart = 0;  # becomes 1 as soon as a motif is printed


$bpoutfilename = $ARGV[0] || die "USAGE: extractBPmotifs.pl bioprospector.out motifwidth motifs.in\n\n";
$width = $ARGV[1]|| die "USAGE: extractBPmotifs.pl bioprospector.out motifwidth motifs.in\n\n";   #motif width
$outfilename =  $ARGV[2]|| die "USAGE: extractBPmotifs.pl bioprospector.out motifwidth motifs.in\n\n";


open(inFile, "$bpoutfilename")|| die "Cannot open input file: $!\n";  # read from bioprospector output file

$emptylines = $width + 3;

open(outFile,">$outfilename")|| die "Specify output file: $!\n"; # write to this file

$motifindex = 0;


push(@consensuslist,"XXXXX");

$line=<inFile>;

$flag = 0;

while ($flag==0)#read in data file one line at a time
  {

   $line=<inFile>;

   if($line =~ m/Total\stime.+?/){
    $flag = 1;
   }

 

 elsif(($line ne "") && ($line ne "\n"))	#not an empty line
  {
    chomp $line;

    if($line =~ m/Motif\s.+?\((.+?)\//){ #contains "Motif"- start of a new list
      
    $newconsensus = $1;
  

    $tmpindex = $motifindex + 1;
    $consflag = 0;

    for($j = 0; $j < $tmpindex; $j++){

      if($newconsensus =~ m/$consensuslist[$j]/){
	$consflag = 1;
      }
    }
      
    if($consflag == 0){ #new motif type

      push(@consensuslist,$newconsensus);
      $motifindex++;
      # find number of segments
      $line=<inFile>;
      $line=<inFile>;
      chomp $line;
      
      ($junk,$NumberOfSegments) =split(/Sites\s/,$line);
      
      #read next (width + 3) lines
      
      
      for($i = 0; $i < $emptylines; $i++){
	$line=<inFile>;
      }
      
      
      #read (NumberOfSegments)  motif names
      
      for($i = 0; $i < $NumberOfSegments; $i++){
	$line=<inFile>;
	chomp $line;
	($Seqno,$jnk1, $jnk2,$jnk3,$jnk4,$dir,$pos) =split(/\s/,$line);
	$Seqno =~ s/>//;

	$motifname=<inFile>;
	chomp $motifname; # motif in next line

	if($dir=~ m/f/){

	    if($beforestart == 1){## at least one motif has been found

		## check for uniqueness before printing

		my $element =  join "\t", $Seqno, $pos; 

		if (grep {$_ eq $element} @comparelist) {
		  ## do nothing, matches an earlier site
		}
		else{
		    print outFile "\n$motifindex\t $Seqno\t $pos\t $motifname";
		    ## add new element to comparelist
		    push(@comparelist,$element);
		}
	    }
	    else{
		
		## start the comparelist
		$str1 =  join "\t", $Seqno, $pos;
		push(@comparelist,$str1);

		print outFile "$motifindex\t $Seqno\t $pos\t $motifname";
		$beforestart = 1;
	    }
	}
	elsif($dir=~ m/r/){ 

	    $newpos =  $pos - $width + 1;
	    
	    if($beforestart == 1){

		## check for uniqueness before printing

		my $element =  join "\t", $Seqno, $pos; 

		if (grep {$_ eq $element} @comparelist) {
		  ## do nothing, matches an earlier site
		}
		else{
		    ## add new element to comparelist
		    push(@comparelist,$element);		    
		    print outFile "\n$motifindex\t $Seqno\t $newpos\t $motifname";
		}
	    }
	    else{
		## start the comparelist
		$str1 =  join "\t", $Seqno, $pos;
		push(@comparelist,$str1);

		print outFile "$motifindex\t $Seqno\t $newpos\t $motifname";
		$beforestart = 1;
	    }
	}
	else { # neither 'f' nor 'r'
	    print "error! unknown direction of strand!\n";
	    exit(1);
	}  
      }
    }
  }
}
}

close(outFile);
close(inFile);

