#################################################################################
# Copyright (C) 2004 Mayetri Gupta. All rights reserved.                        #
# This program is free software; you can redistribute it and/or modify it under #
# the same terms as Perl itself.                                                #
#                                                                               #
# E-mail: gupta@bios.unc.edu                                                    #
#                                                                               #
# Program to extract unique motifs and sites from MEME output and compile       #
# a list in the required input format for the EMC Module Sampler                #
#                                                                               #
# IMPORTANT NOTE: MEME output must be in PLAIN TEXT FASTA FORMAT                #
# Please choose the options " -dna  -print_fasta -text " when running MEME      #
#################################################################################


#!/usr/local/bin/perl
#!c:/perl -w


## Usage: extractMEMEmotifs.pl  meme.in  motifs.out

$infilename = $ARGV[0];
$outfilename = $ARGV[1];

open(inFile,"$infilename")|| die "Cannot open input file: $!\n";

open(outFile,">$outfilename")|| die "Cannot open output file: $!\n"; # write to this file

$motifindex = 1;

$line=<inFile>;
$beforestart = 0;


 $flag = 0;

while ($flag==0){#read in data file one line at a time
     
     $line=<inFile>;
     chomp $line;
		     
# if finds the start of a motif
     if($line =~ m/FASTA.+?/){  # read in all motifs and write to output
	 $line=<inFile>;            # this is a junk line
	 chomp $line;
	 $mflag = 1;  

	 while($mflag == 1){
	     
	     $line=<inFile>;  
	     chomp $line;
	     
	     if($line =~ m/\>(.+?)/){ # contains seq number and pos
		 
		 ($seqno,$jnk) = split(/\s/,$line);
		 $seqno =~ s/>//;
		 ($jnk,$pos) = split(/pos/,$line);

		 $line=<inFile>;  
		 chomp $line; 
		 $motifname = $line;
		 
# write motif index, seq, position and consensus to file
       
		 
		 if($beforestart==0){# no motif yet printed in the list
		     print outFile "$motifindex\t $seqno\t $pos\t $motifname";
		     $beforestart = 1;
					 
		     ## start the comparelist
		     my $str1 =  join "\t", $seqno, $pos;
		     push(@comparelist,$str1);
		 }
		 else{
		     
		     my $element =  join "\t", $seqno, $pos;
		     if (grep {$_ eq $element} @comparelist) {
			 ## do nothing, matches an earlier site
		     }
		     else{
			 push(@comparelist,$element);
		
			 print outFile "\n$motifindex\t $seqno\t $pos\t $motifname";
		     }
		 }
		 

 
	     }#end elsif, next motif, same index
		 
		 elsif($line =~ m/-----------(.+?)/){
		     $mflag = 0;
		     $motifindex++;
		 }
    
	 }#end while flag==1; next motifindex
     } #end if; start reading rest of file till it finds next motif flag

     # check for end of file, then set flag = 1
   if($line =~ m/CPU.+?/){  
       $flag = 1;
   }
# otherwise beginning of loop     
}#end while flag == 0;


$totnum = $motifindex - 1;

close(inFile);
close(outFile);

print "\n total number of motifs found : $totnum\n";

system("sort +0n +1n +2n $outfilename > memeintermed.buf");

open(inFile,"memeintermed.buf")|| die "problem reading from buffer";

open(outFile,">$outfilename")|| die "cannot write to output!";


$line=<inFile>;
chomp $line;
print outFile "$line";


$flag = 0;

while($line=<inFile>){
    chomp $line;
    print outFile "\n$line";
}

close(inFile);
close(outFile);

system("rm -f memeintermed.buf");


